# The LearningOnline Network with CAPA
# User Roles Screen
# (Directory Indexer
# (Login Screen
# 5/21/99,5/22,5/25,5/26,5/31,6/2,6/10,7/12,7/14 Gerd Kortemeyer)
# 11/23 Gerd Kortemeyer)
# 1/14,03/06,06/01,07/22,07/24,07/25,09/04,09/06 Gerd Kortemeyer
#
package Apache::lonroles;

use strict;
use Apache::lonnet();
use Apache::Constants qw(:common);
use Apache::File();

sub handler {
    my $r = shift;
    $r->content_type('text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# ---------------------------------------------------------------- Print Header
    $r->print(<<ENDHEADER);
<html>
<head>
<title>LON-CAPA User Roles</title>
</head>
<body bgcolor="#FFFFFF">
ENDHEADER

# ------------------------------------------ Get Error Message from Environment

    my ($fn,$priv,$nochoose,$error,$msg)=split(/:/,$ENV{'user.error.msg'});
    $r->log_reason(
 "$msg for $ENV{'user.name'} domain $ENV{'user.domain'} access $priv",$fn);

# -------------------------------------------------------- Generate Page Output

    if ($error) {
	$r->print("<h1>LON-CAPA Access Control</h1>");
        $r->print("<hr><pre>Access  : ".
                  Apache::lonnet::plaintext($priv)."\n");
        $r->print("Resource: $fn\n");
        $r->print("Action  : $msg\n</pre><hr>");
    } else {
        $r->print("<h1>LON-CAPA User Roles</h1>");
    }

    my $now=time;
    my $then=$ENV{'user.login.time'};

    if ($nochoose) {
	$r->print("<h2>Assigned User Roles</h2>\n");
    } else {
        $r->print("<h2>Select a User Role</h2>\n");
        $r->print('<form method=get action="'.$fn.'">');
    }

    $r->print('<table><tr><th>User Role</th><th colspan=2>Extent</th>'.
              '<th>Start</th><th>End</th><th>Remark</th></tr>'."\n");
    my $envkey;
    foreach $envkey (sort keys %ENV) {
        if ($envkey=~/^user\.role\./) {
	    my ($dum1,$dum2,$role,@pwhere)=split(/\./,$envkey);
            my $where=join('.',@pwhere);
            my ($tstart,$tend)=split(/\./,$ENV{$envkey});
            my $tremark='';
            my $tstatus='is';
            my $tpstart='&nbsp;';
            my $tpend='&nbsp;';
            if ($tstart) {
		if ($tstart>$then) { 
                   $tstatus='future';
                   if ($tstart<$now) { $tstatus='will'; }
                }
                $tpstart=localtime($tstart);
            }
            if ($tend) {
                if ($tend<$then) { $tstatus='expired'; }
                if ($tend>$now) { $tstatus='will_not'; }
                $tpend=localtime($tend);
            }
            my $tbg;
            if ($tstatus eq 'is') {
		$tbg='#77FF77';
            } elsif ($tstatus eq 'future') {
                $tbg='#FFFF77';
            } elsif ($tstatus eq 'will') {
                $tbg='#FFAA77';
                $tremark.='Active at next login.';
            } elsif ($tstatus eq 'expired') {
                $tbg='#FF7777';
	    } elsif ($tstatus eq 'will_not') {
                $tbg='#AAFF77';
                $tremark.='Expired after logout.';
            }
            my $trole;
            if ($role =~ /^cr\//) {
	       my ($rdummy,$rdomain,$rauthor,$rrole)=split(/\//,$role);
               $tremark.='<br>Defined by '.$rauthor.' at '.$rdomain.'.';
               $trole=$rrole;
	    } else {
               $trole=Apache::lonnet::plaintext($role);
            }
            my $ttype;
            my $twhere;
            my ($tres,$tdom,@trest)=split(/\//,$where);
            if ($where=~/\.course$/) {
		$ttype='Course';
                $twhere=$tdom.'/'.join('/',@trest);
            } elsif ($tdom) {
                $ttype='Domain';
                $twhere=$tdom;
            } else {
                $ttype='System';
                $twhere='/';
            }
               
            $r->print('<tr bgcolor='.$tbg.'><td>'.$trole.'</td><td>'.
		      $ttype.'</td><td>'.$twhere.'</td><td>'.$tpstart.
                      '</td><td>'.$tpend.
                      '</td><td>'.$tremark.'&nbsp;</td></tr>'."\n");
        }
    }

    $r->print('</table>');
    unless ($nochoose) {
	$r->print("</form>\n");
    }
# ----------------------------------------------------------------- Priviledges

    $r->print('<hr><h2>Priviledges</h2>');

    foreach $envkey (sort keys %ENV) {
        if ($envkey=~/^user\.priv\./) {
	    my ($dum1,$dum2,@pwhere)=split(/\./,$envkey);
            my $where=join('.',@pwhere);
            my $ttype;
            my $twhere;
            my ($tres,$tdom,@trest)=split(/\//,$where);
            if ($where=~/\.course$/) {
		$ttype='Course';
                $twhere=$tdom.'/'.join('/',@trest);
            } elsif ($tdom) {
                $ttype='Domain';
                $twhere=$tdom;
            } else {
                $ttype='System';
                $twhere='/';
            }
            $r->print("\n<h3>".$ttype.': '.$twhere.'</h3><ul>');
            map {
              if ($_) {
		  my ($prv,$restr)=split(/\&/,$_);
                  my $trestr='';
                  if ($restr ne 'F') {
                      my $i;
                      $trestr.=' (';
                      for ($i=0;$i<length($restr);$i++) {
		         $trestr.=
                           Apache::lonnet::plaintext(substr($restr,$i,1));
                         if ($i<length($restr)-1) { $trestr.=', '; }
		      }
                      $trestr.=')';
                  }
                  $r->print('<li>'.Apache::lonnet::plaintext($prv).$trestr.
                            '</li>');
	      }
            } sort split(/:/,$ENV{$envkey});
            $r->print('</ul>');
        }
    }

# -------------------------------------------------------------- Debug - remove

    $->print("<hr><h1>Debugging</h1><hr>\n");
    
    foreach $envkey (sort keys %ENV) {
	$r->print("$envkey ---- $ENV{$envkey}<br>");
    }

    $r->print("</body></html>\n");
    return OK;
} 

1;
__END__
